package com.checkpoint.management.groovy_client.useful_scripts

import com.checkpoint.infrastructure.types.CPUUID
import com.checkpoint.management.dlecommon.ngm_api.CpmiOwned
import com.checkpoint.management.groovy_client.Login
import com.checkpoint.management.objects_constants.ObjectsConstants
import com.checkpoint.objects.classes.CpmiDynamicAttribInt
import com.checkpoint.objects.classes.dummy.CpmiDynamicAttrib
import com.checkpoint.objects.ips_classes.CpmiDynAttackDynAttribsContainerInt
import com.checkpoint.objects.ips_classes.dummy.CpmiDynAttackDynAttribsContainer
import com.checkpoint.management.dleserver.coresvc.internal.solr.PostgresUtil
import com.checkpoint.management.groovy_client.ServerServices
import com.checkpoint.management.groovy_client.Session
import com.checkpoint.management.web_services.dleserver.ObjectCrudSvcRemote
import com.checkpoint.management.web_services.dleserver.WorkSessionMgmtSvcRemote
import org.apache.avro.generic.GenericData

/*
This script goes over CpmiDynAttackDynAttribsContainer objects, and remove duplicates (duplicates cause install policy failure).
For more ditails see:  sk118894
To run the script:
In SMC: USERNAME=<SmartConsole Username with admin privileges>PASSWORD=<PASSWORD> $MDS_FWDIR/scripts/run_groovy_script.sh ~/install_policy_failure_due_IPS_objects.groovy
In MDS: USERNAME=<SmartConsole Username with admin privileges>PASSWORD=<PASSWORD> DOMAIN=<domain name> $MDS_FWDIR/scripts/run_groovy_script.sh ~/install_policy_failure_due_IPS_objects.groovy
 */
def fixDuplicatesForDomain(String username, String password, CPUUID domainId)
{
    try {
        try {
            println "GROOVY install_policy_failure_due_IPS_objects: Login to " + domainId + "..."
            Login.doLoginDomain(username, password, domainId);
            println "GROOVY install_policy_failure_due_IPS_objects: Login done..."
        }
        catch (Exception e) {
            e.printStackTrace()
            println "GROOVY PRINT:: Failed to login ::" + e
        }
        finally {
            println "GROOVY PRINT:: End login ::"
        }

        println "GROOVY install_policy_failure_due_IPS_objects: Query...";
        PostgresUtil postgresUtil = new PostgresUtil();
        String postgresQuery = "select objid from dleobjectderef_data where objclass = 'com.checkpoint.objects.ips_classes.dummy.CpmiDynAttackDynAttribsContainer' and domainid='" + domainId + "' and dlesession = 0 and not deleted;"
        List<Object[]> postgresResultsObjsIds = postgresUtil.getAnyObjectByQuery(postgresQuery)
        println "GROOVY install_policy_failure_due_IPS_objects: Query done...";
        //println "GROOVY install_policy_failure_due_IPS_objects: Query results " + postgresResultsObjsIds.toString();


        ObjectCrudSvcRemote crudSvc = (ObjectCrudSvcRemote) ctx.getBean("objectCrudSvcRemote");

        println "GROOVY install_policy_failure_due_IPS_objects: processing...";
        List<CpmiDynamicAttrib> objsForDeletion = new ArrayList<>();
        for (Object[] objID : postgresResultsObjsIds) {
            String objectIdStr = objID[0]; // got objid:0000000f-00e4-0048-8376-cd92d67e0c1b
            String cleanObjId = objectIdStr.substring(6); // got "0000000f-00e4-0048-8376-cd92d67e0c1b"
            CPUUID objId = CPUUID.fromString(cleanObjId);  // got CPUUID of "0000000f-00e4-0048-8376-cd92d67e0c1b"
            println "GROOVY install_policy_failure_due_IPS_objects: got objId " + objId.toString();
            CpmiDynAttackDynAttribsContainerInt obj1 = crudSvc.getObject(objId);
            List<CpmiOwned> attrsList = obj1.getAttributes();
            for (CpmiOwned attr : attrsList) {
                CpmiDynamicAttribInt attrObj = (CpmiDynamicAttribInt) attr;
                if (attrObj.getAttribId() == 0L) {
                    println "GROOVY install_policy_failure_due_IPS_objects: add bad attribute to deletion list ..." + attrObj.getObjId().toString();
                    objsForDeletion.add(attrObj);
                }
            }
            if (objsForDeletion.size() > 0) {
                println "GROOVY install_policy_failure_due_IPS_objects: removing from " + attrsList.size() + " attribs";
                println "GROOVY install_policy_failure_due_IPS_objects: removing " + objsForDeletion.size() + " bad attributes";
                for (CpmiOwned objForDeletion : objsForDeletion) {
                    attrsList.remove(objForDeletion);
                }
                println "GROOVY install_policy_failure_due_IPS_objects: now we have " + attrsList.size() + " attribs";
                List<CpmiOwned> newAttrList = new ArrayList<>();
                for (CpmiOwned newAttr : attrsList) {
                    newAttrList.add(newAttr);
                }
                println "GROOVY install_policy_failure_due_IPS_objects: newAttrList has " + newAttrList.size() + " size";
                obj1.getAttributes().clear();
                println "GROOVY install_policy_failure_due_IPS_objects: obj1.getAttributes() had before - " + obj1.getAttributes().size() + " size";
                obj1.setAttributes(newAttrList);
                println "GROOVY install_policy_failure_due_IPS_objects: obj1.getAttributes() has now - " + obj1.getAttributes().size() + " size";
                println "GROOVY install_policy_failure_due_IPS_objects: updating object " + obj1.getObjId().toString();
                crudSvc.updateObject(obj1);
                objsForDeletion.clear();
            }
        }

        println "GROOVY install_policy_failure_due_IPS_objects: Publish...";
        WorkSessionMgmtSvcRemote workSessionMgmtSvc = (WorkSessionMgmtSvcRemote) ctx.getBean("workSessionMgmtSvcRemote");
        workSessionMgmtSvc.publish();
        println "GROOVY install_policy_failure_due_IPS_objects: Done publish";
    }
    catch (Exception e) {
        e.printStackTrace()
        println "GROOVY install_policy_failure_due_IPS_objects: Failed ::" + e
    }
    finally {
        Session.logout();
        println "GROOVY install_policy_failure_due_IPS_objects: Logged out ;-)";
    }

}

println "GROOVY PRINT:: Start."
ServerServices.init(ctx);
try {

    def env = System.getenv()
    Boolean isMds = false;
    try
    {
        "mdsstat".execute().text
        println "GROOVY install_policy_failure_due_IPS_objects: MDS"
        isMds = true;
    }
    catch (Exception e) {
        println "GROOVY install_policy_failure_due_IPS_objects: not a MDS"
    }

    if(isMds)
    {
        println "GROOVY install_policy_failure_due_IPS_objects: Query for domain id...";
        PostgresUtil postgresUtil = new PostgresUtil();
        String postgresQuery = "select objid from domainbase_data where name='" + env['DOMAIN'] + "' and dlesession=0 and not deleted;"
        Object[] domainQueryRes = postgresUtil.getAnyObjectByQuery(postgresQuery)[0];// got objid:0000000f-00e4-0048-8376-cd92d67e0c1b
        String domainIdStr = domainQueryRes[0]
        String cleanDomainIdStr = domainIdStr.substring(6); // got "0000000f-00e4-0048-8376-cd92d67e0c1b"
        CPUUID domainId = CPUUID.fromString(cleanDomainIdStr);  // got CPUUID of "0000000f-00e4-0048-8376-cd92d67e0c1b"
        println "GROOVY install_policy_failure_due_IPS_objects: got domain id:" + cleanDomainIdStr;
        fixDuplicatesForDomain(env['USERNAME'], env['PASSWORD'],domainId)
        fixDuplicatesForDomain(env['USERNAME'], env['PASSWORD'],ObjectsConstants.Domains.GLOBAL_DOMAIN)
    }
    else
    {
        fixDuplicatesForDomain(env['USERNAME'], env['PASSWORD'],ObjectsConstants.Domains.SMC_USER_DOMAIN)
    }
}
catch (Exception e) {
    e.printStackTrace()
    println "GROOVY install_policy_failure_due_IPS_objects: Failed ::" + e
}

println "GROOVY install_policy_failure_due_IPS_objects: GROOVY PRINT:: End ;-)"


